package MobileMonitoring;

import StaticMonitoring.Settings;
import MonitorConfig.snmpTable;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.utility.*;
import com.adventnet.snmp.snmp2.*;
import java.io.PrintStream;
import java.util.*;
import java.net.*;

public class SnmpTableMA extends SnmpV3MobileManager
{
    //OID associato alla tabella oggetto della richiesta
    String tab;
    //OIDs delle colonne della tabella PRIVATE del PREFISSO COMUNE (tab)
    //es. .1.3.6.1.2.1.2.2.1.1 --> 1.1
    String[] columns;
    //numero di colonne
    int Ncol;
    //vettore di tabelle risultato
    //Vector di snmpTable job_result;

    public void putArgument(Object o)
    {
        InfoTable Info = (InfoTable) o;
        super.putArgument(Info);
        tab = Info.getTab();
        columns = Info.getCols();
        Ncol = Info.getNCol();
    }

    /**questo  il metodo di partenza, eseguito all'attivazione dell'agente:
    //qui si avvia il cronometro che conter il tempo necessario al completamento
    //dell'operazione, si definir la prima tappa del "viaggio" e si inizializzer
    //il campo home che dovr poi essere utilizzato per il ritorno al Place
    //di partenza
    */
    public void run()
    {
        super.run();
    }

    /**metodo di passaggio ad una tappa successiva,  qui che si controlla se il viaggio
    // finito, in tal caso si torna al Place di partenza
    */
    public void goNext()
    {
        super.goNext();
    }

    //qui sono racchiuse le operazioni necessarie all'adempimento del compito
    //specifico di questo Agente Mobile
    public void doLocalJob(Settings sets,SnmpSession session)
    {
        PrintStream out = agentSystem.getOut();
        snmpTable localtab = getTable(sets,session);
        result_code[current] = true;
        job_result.add(current,localtab);

        session.close();
        api.close();
		if (++current != Ntargets)
	        nexthop = targets[current];
        goNext();
    }

    //nel caso mobile la getTable e' diversa perche' non si devono caricare i moduli MIB
    //e' gia' stato fatto nel manager stazionario, cosi' ora si ha gia' la conoscenza dei
    //nomi delle colonne da richiedere
    snmpTable getTable(Settings sets,SnmpSession session)
    {
        PrintStream out = agentSystem.getOut();
        boolean next = true;
        SnmpPDU ris = new SnmpPDU();
        snmpTable table = new snmpTable(tab,Ncol,debug,out);
        //la prima volta che faccio una richiesta devo ricostruire gli OID totali
		String[] totalOID = getCols();

        while (next)
        {
            ris = snmpOp(sets,session,totalOID);
            next = (check(ris));
            if (next)
            {
                table.addRow(ris);
              //  System.out.println("NEXT OIDs");
                //recupero gli OID della risposta per utilizzarli nella prossima richiesta
                for(int i=0; i<totalOID.length; i++)
                {
                    totalOID[i] = ris.getObjectID(i).toString();
                    //System.out.println(((mibops.getMibNode(ris.getObjectID(i))).getOIDString())+"   "+OIDs[i]);
                }
            }
            else //errore o risposta non piu' relativa alla tabella
            {
                next = false;
                break;
            }
        }

        return table;
	}

    //restituisce la lista degli object identifier completi delle colonne della tabella
    //es. oidtab = .1.3.6.1.2.1.2.2    oidcol = 1.1
    public String[] getCols()
    {
        String[] ret = new String[Ncol];
        for (int j=0; j< Ncol; j++)
            ret[j] = tab+"."+columns[j];
        return ret;
    }

    //verifica se la PDU e' OK
    //pdu e' la pdu ottenuta in risposta ad una richiesta SNMP
    //restituisce true se la pdu e' OK
    //false se e' nulla o se contiene indicazioni di errore o se anche una sola variabile
    //della PDU non appartiene alla tabella

    boolean check(SnmpPDU pdu)
    {
        PrintStream out = agentSystem.getOut();

        //verifico se la pdu e' nulla (time out)
        if (pdu == null)
        {
            trace_err(out,getHeader()+" NULL PDU in response");
            return false;
        }

        //verifico se la pdu contiene errori
        if (pdu.getErrstat() > 0)
        {
            trace_err(out,getHeader()+" Error occurred in PDU "+pdu.getError());
            pdu.fix();
            return false;
        }

        //verifico se nella pdu di risposta sono contenuti elementi appartenenti alla tabella
        int nvar = (pdu.getVariableBindings()).size();
        int j = 0;
        boolean next = true;

        while ((next)&&(j<nvar))
            next = pdu.getObjectID(j++).toString().startsWith(tab+".");

       return next;
    }

    //anche snmpOp puo' essere alleggerito visto che si trattera' sempre di una GETNEXT
    //percio' l'interfaccia e' leggermente diversa da quella del caso stazionario
     SnmpPDU snmpOp(Settings sets,SnmpSession session,String[] OIDs)
     {
        PrintStream out = agentSystem.getOut();
        //creo una nuova PDU che dovro' poi spedire
        SnmpPDU pdu = new SnmpPDU();
        //setto il tipo di operazione nella pdu
        pdu.setCommand(SnmpAPI.GETNEXT_REQ_MSG);
		
        //introduco tutti gli OID richiesti nella PDU
        for (int j=0; j < OIDs.length; j++)
        {
            SnmpOID oid = new SnmpOID(OIDs[j]);
            if (oid.toValue() == null)
            {
                trace_err(out,getHeader()+" Invalid OID argument: "+OIDs[j]);
                return null;
            }
            else pdu.addNull(oid);
        }

		pdu.setUserName(sets.userName.getBytes());
		if (session.getVersion() == SnmpAPI.SNMP_VERSION_3)
		{
			if (sets.config == false)
			USMUtils.init_v3_params(sets.userName,sets.authProto,sets.authPwd,sets.privPwd,session.getPeername(),session.getRemotePort(),session);
			pdu.setContextName(sets.contextName.getBytes());
			pdu.setContextID(sets.contextID.getBytes());
		}

        try  { pdu = session.syncSend(pdu); }
        catch (SnmpException e)
        {
            trace_err(out,getHeader()+" Error sending PDU"+e.getMessage());
            return null;
        }

        return(pdu);
   }

   //N.B. non prevedendo MIB files per motivi di efficienza, la stampa che puo' fare
   //agente mobile e' molto meno significativa di quella che potra' fare il manager stazionario
   //all'atto della ricezione dei dati
   void ShowJob(PrintStream o)
   {
       o.println("Data retrieved from "+getID().toString());
       for (int j=0; j<Ntargets; j++)
       {
           o.println(targets[j].toString());
           snmpTable cur = (snmpTable)job_result.elementAt(j);
           if (cur == null)
           		o.println("Null result");
           else
           		cur.printTable(o);
       }
   }

}
