package MobileMonitoring;

import StaticMonitoring.Settings;
import SOMA.*;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.utility.*;
import com.adventnet.snmp.snmp2.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class SnmpV3MobileManager extends SnmpMobileManager
{
    //nomi di sicurezza con cui l'Agente Mobile deve di volta in volta presentarsi
    //agli Agenti Snmp locali
    String[] securityNames;
    //file da cui deserializzare la configurazione SnmpV3
    //in realta' potrei pensare ad un nome diverso per ogni place
    File serFile;

    /**qui si recuperano le singole informazioni a partire dall'oggetto passato
    //in input e si inizializzano gli altri attributi.
    //o e' un oggetto della classe DrivePlan
    */
    public void putArgument(Object o)
    {
        InfoBasedOnConfig Info = (InfoBasedOnConfig) o;
        super.putArgument(Info);
        securityNames = Info.getSecNames();
        serFile = Info.getFile();
    }

    /**metodo di preparazione dell'ambiente locale ai fini dell'adempimento
    //del compito: creazione thread di gestione delle sessioni Snmp, creazione
    //di una sessione Snmp per l'interazione con l'Agente Snmp presente localmente
    //non si prevede recupero di informazioni da configurazione
    */
    public void prepareLocalJob()
    {
        PrintStream out = agentSystem.getOut();
        String location = agentSystem.getPlaceID().toString();
        //variabile destinata a contenere i settaggi da fare sulla sessione
        Settings sets = null;
        if ( !(location.equals(targets[current].toString())) )
        {
            trace_err(out,getHeader()+" Wrong Target. Right is: "+targets[current]);
            result_code[current] = false;
            job_result.add(current,null);
            if (++current != Ntargets)
	            nexthop = targets[current];
            goNext();
        }
        else
        {
            int version = snmpVs[current];
            int port = snmpPorts[current];

            String params[] = new String[Settings.NMAXOPTIONS];
            params[Settings.SECNAME] = securityNames[current];
            //setto i parametri "remoti": indirizzo,porta,versione dell'agente Snmp
            String host = "127.0.0.1";
            params[Settings.PORT] = Integer.toString(port);
            if (version == SnmpAPI.SNMP_VERSION_1)
            	params[Settings.VERSION] = "v1";
            else if (version == SnmpAPI.SNMP_VERSION_2C)
            	params[Settings.VERSION] = "v2";
            else
            	params[Settings.VERSION] = "v3";
//out.println(params[Settings.VERSION]+" "+params[Settings.PORT]+" "+params[Settings.SECNAME]);
            api = new SnmpAPI();
        	api.start();
            SnmpSession session = new SnmpSession(api);
            //setto i parametri locali: indirizzo (localhost) e porta (la prima libera assegnata dal sistema)
            //da cui inviare le richieste all'agente Snmp.. in realta' e' gia' cosi' il default
            //String[] addrs = new String[1];
            //addrs[0] = "127.0.0.1":q
            //session.setLocalAddresses(addrs);
            //session.setLocalPort(0);
            try {session.open();}
            catch(SnmpException ex) {trace_err(out,getHeader()+" Error opening the session");}
            //se si tratta di una richiesta V3 deserializzo le informazioni 
            //poi eseguo i settaggi sulla sessione tramite Settings
            if (version == SnmpAPI.SNMP_VERSION_3)
            {
//out.println("Ramo sbagliato dell'if");
                api.setSerializeFileName(serFile.getAbsolutePath());
                try
                {
                    if (api.deSerialize() == false)
                    {
                        trace_err(out,getHeader()+" Deserialization Problem Occurring");
                    	result_code[current] = false;
                    	job_result.add(current,null);
                    	if (++current != Ntargets)
	                    	nexthop = targets[current];
                    	goNext();
                    }
                }

                catch(SnmpException ex) { result_code[current] = false;
                   	job_result.add(current,null);
                   	if (++current != Ntargets)
                    	nexthop = targets[current];
                    goNext();}
                sets = new Settings(session,host,params,true,debug,out);
            }
            //altrimenti effettuo subito i settaggi
            else
                sets = new Settings(session,host,params,false,debug,out);

            doLocalJob(sets,session);
     }
    }
    
    //qui sono racchiuse le operazioni necessarie all'adempimento del compito
    //specifico di questo Agente Mobile
    public void doLocalJob(Settings sets,SnmpSession session)
    {
		session.setUserName(sets.userName.getBytes());
		if (session.getVersion() == SnmpAPI.SNMP_VERSION_3)
		{
			if (sets.config == false)
			USMUtils.init_v3_params(sets.userName,sets.authProto,sets.authPwd,sets.privPwd,session.getPeername(),session.getRemotePort(),session);
			session.setContextName(sets.contextName.getBytes());
			//session.setContextID(sets.contextID.getBytes());
		}
        String res = checkSys(session);
        job_result.add(current,res);
        result_code[current] = true;

        session.close();
        api.close();

		if (++current != Ntargets)
	        nexthop = targets[current];
        goNext();

    }
    
    //metodo di visualizazione dei risultati su un particolare PrintStream
    void ShowJob(PrintStream o)
    {
        o.println(getHeader()+" Data retrieved from "+getID().toString());
        for (int j=0; j<Ntargets; j++)
            o.println(targets[j].toString()+":"+(String)job_result.elementAt(j));
    }

}