package MonitorConfig;

import SOMA.*;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.naming.place.*;
import SOMA.naming.domain.*;

import com.adventnet.snmp.mibs.*;
import com.adventnet.snmp.snmp2.*;
import java.io.*;
import java.util.*;

public class Configuration
{
    private PlaceNameService PNS = null;
    private DomainNameService DNS = null;
    public SnmpAPI api = null;
    private SnmpSession discoverySession = null;

	//var static destinata a mantenere il prefisso simbolico standard da utilizzare
	//in caso di OID simbolici relativi (name1.name2...)
   static String standard_sym_prefix = ".iso.org.dod.internet.mgmt.mib-2.";

   //variabili che indicano se si e' in modalita' di debugging (output per errori)
    //e quali PrintStream utilizzare
    private boolean debug = false;
    //tengo un attributo che mi indichi se e' gia' stata fatta deserializzazione oppure no
    private boolean deserialized = false;
    private PrintStream out;

    //nome del file di serializzazione di userTable
    String USMserFile = "USMser.ser";
    //nome del file di serializzazione di engineTable (e' assegnato da AdventNet)
    String EngineFile = "EngineEntry.ser";
    //nome del file di serializzazione di MIBsNodes
    String MIBserFile = "SuppMIBs.ser";
    //da togliere dopo aver fatto le prove su piu' hosts
    String AgentserFile = "AgentSnmp.ser";
    //direttorio in cui trovare i MIB files in caso di caricamento
    String MIBsFiledir = null;

    //tabella di configurazione dei motori SNMPv3
    public SnmpEngineTable engineTable = null;
    //tabella di configurazione degli utenti SNMPv3
    public USMUserTable userTable = null;
    public MibOperations mibops;

    //struttura nella quale verranno memorizzate le associazioni
    //motore Snmp (cioe' indirizzo:porta --> mib supportati) ed il nome assoluto del
    //direttorio locale nel quale trovare i MIB files corrispondenti
    public SupportedNodesMIB MIBsNodes = null;

    public InfosAgentSnmp agentSnmp = null;
    
    public StatusDomains statD = null;

    public Configuration(PlaceNameService pns, DomainNameService dns, boolean deb, PrintStream o)
    {
        debug = deb;
        out =o;
        PNS = pns;
        DNS = dns;
        mibops = new MibOperations();
        if (dns != null)
	        statD = new StatusDomains(dns.getDomainsArray());
        MIBsNodes = new SupportedNodesMIB(MIBsFiledir,MIBserFile,debug,out);
        agentSnmp = new InfosAgentSnmp(AgentserFile,debug,out);
    }

    public void startConfig(SnmpAPI Api, SnmpSession discovery, boolean deser)
    {
        api = Api;
        discoverySession = discovery;
        if (deser)
        {
        	api.setSerializeFileName(USMserFile);
	        try
    	    {
        	    if (api.deSerialize() == false)
        	    {
            	    trace_err("Error restoring V3 Configuration Tables from: "+api.getSerializeFileName());
            	    return;
            	}
	            else
    	        {
        	        userTable = api.getUSMTable();
            	    engineTable = api.getSnmpEngine();
            	}
            	if (new File("EngineEntry.ser").exists())
                	engineTable.deSerialize();
	            if ((MIBsNodes.deSerialize()== false) || (agentSnmp.deSerialize() == false))
	            {
    	            trace_err("Error restoring SupportedMIBNodes or InfosAgentSnmp");
    	            return;
    	        }
        	}
        	catch(SnmpException e) {
        		trace_err("Exception during deserialization");
        		e.printStackTrace();
        		return;}

        	deserialized = deser;
        }
        else
        {
           userTable = api.getUSMTable();
 	       engineTable = api.getSnmpEngine();
 	    }
    }

    public void stopConfig()
    {
    	//serializzo solo se durante l'attivita' del StaticManager e' stato settato il
    	//file di serializzazione
        if (api.getSerializeFileName() != null)
        {
            try
            {
                if (api.serialize()== false)
                    trace_err("Error storing V3 Configuration Tables");

                if (engineTable == null)
                	out.println("Err");

                engineTable.serialize();
                MIBsNodes.serialize();
                agentSnmp.serialize();
            }
            catch(SnmpException e) {trace_err("Exception during serialization");}
        }

    }

    public void setDebug(boolean status)
    {
        debug = status;
        MIBsNodes.setDebug(status);
        agentSnmp.setDebug(status);
    }

    public boolean isDebug()
    {
        return debug;
    }

    public PrintStream getOut()
    {
        return out;
    }

    public void setOut(PrintStream ps)
    {
        out = ps;
    }

    void trace_err(String s)
    {
        String current_class = (getClass().getName())+">";
        if (debug)
            out.println(current_class+s);
    }


	void trace(String s)
	{
		out.println(s);
	}

	public String getUSMserFile()
	{
		return USMserFile;
	}

	public void setUSMserFile()
	{
		api.setSerializeFileName(USMserFile);
	}

    public void setUSMserFile(String ser)
    {
        api.setSerializeFileName(ser);
        USMserFile = new String(ser);
    }

//FUNZIONI di CONFIGURAZIONE UTENTI ed ENGINE V3

    //aggiunge un utente alla tabella degli utenti USM e,se non gia' presente, aggiunge anche
    //il motore Snmp ad esso associato alla tabella dei motori Snmp
    //name e' il nome di sicurezza dell'utente
    //remhost e' l'host remoto su cui si trova l'agente Snmp a cui l'utente ha accesso
    //remport e' la porta d'ascolto dell'agente Snmp remoto
    //restituisce true se l'utente e' stato aggiunto con successo alla configurazione
    //            false se l'utente e' gia' presente nella configurazione, cioe' se c'e' gia'
    //                  un'entry con chiave (name,remhost,remport)
    public boolean addUser(String name,String remhost,int remport)
    {
        SnmpEngineEntry snmpEntry;
        //se l'entry corrispondente a (remhost,remport) non e' gia' presente nella tabella dei
        //motori Snmp, la aggiungo
        if ((snmpEntry = engineTable.getEntry(remhost,remport)) == null)
        {
            snmpEntry = new SnmpEngineEntry(remhost,remport);
            snmpEntry.discoverEngineID(discoverySession);
            engineTable.addEntry(snmpEntry);
        }
        byte[] engineID = snmpEntry.getEngineID();
        USMUserEntry entry = new USMUserEntry(name.getBytes(),engineID);
        entry.setSecurityLevel(Snmp3Message.NO_AUTH_NO_PRIV);
        return(userTable.addEntry(entry));
    }

	public boolean addUser(String name,PlaceID placeID,int remport)
	{
        String remhost = PlaceToHost(placeID);
        return(addUser(name,remhost,remport));
    }


    //aggiunge un utente alla tabella degli utenti USM e,se non gia' presente, aggiunge anche
    //il motore Snmp ad esso associato alla tabella dei motori Snmp
    //name e' il nome di sicurezza dell'utente
    //auth_proto e' il protocollo da utilizzare per autenticare l'utente
    //authpwd e' la password di autenticazione dell'utente (n.b. non e' localizzata)
    //remhost e' l'host remoto su cui si trova l'agente Snmp a cui l'utente ha accesso
    //remport e' la porta d'ascolto dell'agente Snmp remoto
    //restituisce true se l'utente e' stato aggiunto con successo alla configurazione
    //            false se l'utente e' gia' presente nella configurazione, cioe' se c'e' gia'
    //                  un'entry con chiave (name,remhost,remport)

    public boolean addUser(String name,String auth_proto,String authpwd,String remhost,int remport)
    {
       /* SnmpEngineEntry snmpEntry;
        //se l'entry corrispondente a (remhost,remport) non e' gia' presente nella tabella dei
        //motori Snmp, la aggiungo
        if ((snmpEntry = engineTable.getEntry(remhost,remport)) == null)
        {
            snmpEntry = new SnmpEngineEntry(remhost,remport);
            snmpEntry.discoverEngineID(discoverySession);
  System.out.println("EngineID: "+new String(snmpEntry.getEngineID()));
            engineTable.addEntry(snmpEntry);
        }

        byte[] engineID = snmpEntry.getEngineID();
        USMUserEntry entry = new USMUserEntry(name.getBytes(),engineID);
        if (auth_proto == "SHA");
            entry.setAuthProtocol(USMUserEntry.SHA_AUTH);
        if (auth_proto == "MD5")
            entry.setAuthProtocol(USMUserEntry.MD5_AUTH);
        else
        {
            trace_err("Authentication protocols supported: MD5,SHA");
            return false;
        }
        entry.setAuthPassword(authpwd.getBytes());
        byte[] authKey = USMUtils.password_to_key(entry.getAuthProtocol(),entry.getAuthPassword(),(authpwd.getBytes()).length,engineID);
        entry.setAuthKey(authKey);
        //entry.setAuthKey(authpwd.getBytes());
        entry.setSecurityLevel(Snmp3Message.AUTH_NO_PRIV);
        return(userTable.addEntry(entry));*/
        
		int authproto = USMUserEntry.NO_AUTH;
		if (auth_proto == "SHA")
   			authproto = USMUserEntry.SHA_AUTH;
		else if (auth_proto == "MD5")
  			authproto = USMUserEntry.MD5_AUTH;
		else
        {
            trace_err("Authentication protocols supported: MD5,SHA");
            return false;
        }
        
		String privPwd = new String("");
    	discoverySession.setPeername(remhost);
		discoverySession.setRemotePort(remport);
		USMUtils.init_v3_params(name,authproto,authpwd,privPwd,discoverySession.getPeername(),discoverySession.getRemotePort(),discoverySession);
		return true;
    }

    public boolean addUser(String name,String auth_proto,String authpwd,PlaceID placeID,int remport)
    {
        String remhost = PlaceToHost(placeID);
        return(addUser(name,auth_proto,authpwd,remhost,remport));
    }

    //aggiunge un utente alla tabella degli utenti USM e,se non gia' presente, aggiunge anche
    //il motore Snmp ad esso associato alla tabella dei motori Snmp
    //name e' il nome di sicurezza dell'utente
    //auth_proto e' il protocollo da utilizzare per autenticare l'utente
    //authpwd e' la password di autenticazione dell'utente (n.b. non e' localizzata)
    //privpwd e' la password da utilizzare per decrittare i messaggi provenienti dall'utente
    //remhost e' l'host remoto su cui si trova l'agente Snmp a cui l'utente ha accesso
    //remport e' la porta d'ascolto dell'agente Snmp remoto
    //restituisce true se l'utente e' stato aggiunto con successo alla configurazione
    //            false se l'utente e' gia' presente nella configurazione, cioe' se c'e' gia'
    //                  un'entry con chiave (name,remhost,remport)
    public boolean addUser(String name,String auth_proto,String authpwd,String privpwd,String remhost,int remport)
    {
		int authproto = USMUserEntry.NO_AUTH;
        if (auth_proto == "SHA");
            authproto = USMUserEntry.SHA_AUTH;
        if (auth_proto == "MD5")
            authproto = USMUserEntry.MD5_AUTH;
        else
        {
            trace_err("Authentication protocols supported: MD5,SHA");
            return false;
        }
    	discoverySession.setPeername(remhost);
		discoverySession.setRemotePort(remport);
        USMUtils.init_v3_params(name,authproto,authpwd,privpwd,discoverySession.getPeername(),discoverySession.getRemotePort(),discoverySession);
        return true;
    }

    public boolean addUser(String name,String auth_proto,String authpwd,String privpwd,PlaceID placeID,int remport)
    {
        String remhost = PlaceToHost(placeID);
        return(addUser(name,auth_proto,authpwd,privpwd,remhost,remport));
    }


    //cancella l'associazione name --> (remhost,remport) dalla configurazione degli utenti
    //(N.B. non il motore Snmp cui era associato)
    //name e' il nome dell'utente
    //remhost e' l'indirizzo IP del motore Snmp remoto
    //remport e' la porta d'ascolto del motore Snmp remoto
    public void delUser(String name,String remhost,int remport)
    {
        SnmpEngineEntry engineEntry = engineTable.getEntry(remhost,remport);
        byte[] engineID = engineEntry.getEngineID();
        userTable.removeEntry(name.getBytes(),engineID);
        return;
    }

	public void delUser(String name,PlaceID placeID,int remport)
	{
        String remhost = PlaceToHost(placeID);
        delUser(name,remhost,remport);
    }


	//da togliere o cambiare perche' non stampa tutte le informazioni
	public void showConfiguredV3Users()
	{
		out.println("UserName SecLevel EngineID");
		for(Enumeration e = userTable.getEnumeration();e.hasMoreElements();)
		{
			USMUserEntry entry = (USMUserEntry)e.nextElement();
			String user = new String(entry.getUserName());
			byte sec = entry.getSecurityLevel();
			String secout = new String("");	;
			if (sec == Snmp3Message.NO_AUTH_NO_PRIV)
				secout = new String("NO_AUTH_NO_PRIV");
			else if (sec == Snmp3Message.AUTH_PRIV)
				secout = new String("AUTH_PRIV");
			else if (sec == Snmp3Message.AUTH_NO_PRIV)
				secout = new String("AUTH_NO_PRIV");
			String engine = new String(entry.getEngineID());
			out.println(user+" "+secout+" "+engine);
		}
	}

	public Enumeration getConfiguredUsers()
	{
		return (userTable.getEnumeration());
	}

    //aggiunge un motore Snmp
    //remhost e' l'indirizzo IP del motore Snmp remoto
    //remport e' la porta d'ascolto del motore Snmp remoto
    //restituisce true se il motore viene aggiunto con successo
    //            false se il motore fa gia' parte della configurazione
    public boolean addEngine(String remhost,int remport)
    {
        if (engineTable.getEntry(remhost,remport) == null)
        {
            SnmpEngineEntry newEntry = new SnmpEngineEntry(remhost,remport);
            newEntry.discoverEngineID(discoverySession);
            return(engineTable.addEntry(newEntry));
        }
        //gia' presente
        return true;
    }

	public boolean addEngine(PlaceID placeID,int remport)
	{
        String remhost = PlaceToHost(placeID);
        return(addEngine(remhost,remport));
    }


    //cancella un motore Snmp dalla tabella dei motori noti e, se esiste un'entry
    //corrispondente anche nella tabella dei MIB files supportati dai nodi, la toglie
    //remhost e' l'indirizzo IP del motore Snmp remoto
    //remport e' la porta d'ascolto del motore Snmp remoto
    public void delEngine(String remhost,int remport)
    {
        engineTable.removeEntry(remhost,remport);
        MIBsNodes.removeEntry(remhost,remport);
        return;
    }

	public void delEngine(PlaceID placeID,int remport)
	{
        String remhost = PlaceToHost(placeID);
        delEngine(remhost,remport);
    }


	public void showConfiguredEngines()
	{
		out.println("RemoteHost RemotePort EngineID");
		for(Enumeration e = engineTable.getEnumeration(); e.hasMoreElements();)
		{
			SnmpEngineEntry entry = (SnmpEngineEntry) e.nextElement();
			out.println(entry.getRemoteHost()+" "+entry.getRemotePort()+" "+new String(entry.getEngineID()));
		}
	}

//FUNZIONI di CONFIGURAZIONE dei MIB SUPORTATI dagli AGENTI SNMP
//in realta' potrei usare direttamente i metodi di SupportedNodesMIB sulla variabile
//MIBsNodes

    /*setta il direttorio del file system locale nel quale sono memorizzati i MIB files
    //abs_dir e' il path assoluto del direttorio
    //ritorna true se il settaggio  avvenuto con successo
    //        false se ci sono stati problemi (non  una directory o non esiste)
    */
    public boolean setDirMibs(String abs_dir)
    {
        if (MIBsNodes.setAbsDir(abs_dir))
        {
        	MIBsFiledir = abs_dir;
            return true;
        }
        trace_err(abs_dir+": directory non esistente o nome non corrispondente a directory");
        return false;
    }

	/** carica un modulo MIB dalla directory specificata con setDirMibs
	*   o, se non e' ancora stata settata, dalla directory corrente
	*/
	public boolean loadMIB(String MIBname)
	{
		if (MIBsFiledir != null)
			try{ mibops.loadMibModule(MIBsFiledir+MIBname);	}
			catch(Exception ex)
			{ trace_err("Error loading MIB file: "+MIBname+" from directory "+ MIBsFiledir);
			   return false;}
		else //cerco nella dir corrente
			try{ mibops.loadMibModule(MIBname);	}
			catch(Exception ex)
			{ trace_err("Error loading MIB file: "+MIBname+" from current directory");
			  return false;}
	    return true;
	}

    //aggiunge una nuova entry alla tabella dei mibs supportati dai motori Snmp
    //remhost e' l'indirizzo del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //mib e' la lista dei moduli MIB supportati dal motore Snmp
    public void addMibsNode(String remhost,int remport, String mibs[])
    {
        MIBsNodes.addEntry(remhost,remport,mibs);
    }

	public void addMibsNode(PlaceID place, int remport, String mibs[])
	{
        String remhost = PlaceToHost(place);
		addMibsNode(remhost,remport,mibs);
	}

    //rimuove l'entry corrispondente ad un motore Snmp specifico dalla tabella
    //dei mib supportati da ogni motore
    //remhost e' l'indirizzo del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    public void removeMibsNode(String remhost,int remport)
    {
        MIBsNodes.removeEntry(remhost,remport);
    }

	public void removeMibsNode(PlaceID place, int remport)
	{
        String remhost = PlaceToHost(place);
		removeMibsNode(remhost,remport);
	}

    //modifica un'entry gia' esistente nella tabella dei mibs supportati
    //dai motori Snmp
    //remhost e' l'indirizzo del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //new_mibs e' la nuova lista di mib supportati da associare al motore Snmp
    public void modifyMibsNode(String remhost,int remport,String new_mibs[])
    {
        MIBsNodes.modifyEntry(remhost,remport,new_mibs);
    }

	public void modifyMibsNode(PlaceID place, int remport, String new_mibs[])
	{
        String remhost = PlaceToHost(place);
		modifyMibsNode(remhost,remport,new_mibs);
	}

    public String[] getSupportedMIBs(String host,int port)
    {
        return(MIBsNodes.getEntry(host,port));
    }

    public String[] getSupportedMIBs(PlaceID place, int port)
	{
        String remhost = PlaceToHost(place);
		return(getSupportedMIBs(remhost,port));
	}

    public void showSupportedMibStructure()
    {
    	MIBsNodes.showContent();
    }

    //metodo per la ricerca e l'eventuale caricamento del nodo MIB
    //contenente la tabella specificata
    public MibNode getloadMibTable(String IP,int port,String oid_tab)
    {
        String supported[] = MIBsNodes.getEntry(IP,port);
        //caso in cui non c'e' un'entry corrispondente nella tabella dei MIB supportati
        //cerco tra quelli caricati
        if (supported == null)
        	return(getloadMibTable(oid_tab));
        	
        for (int j=0; j<supported.length; j++)
        {
            //recupera il modulo supportato
            MibModule module = mibops.getMibModule(supported[j]);
            //caso in cui il modulo non e' ancora stato caricato
            if (module == null)
            try
            {
                module = mibops.loadMibModule(MIBsNodes.getAbsDir()+supported[j]);
            }
            catch(Exception ex)
            {
                trace_err("Error loading mib: "+ex);
                return null;
            }
            //recupera il nodo corrispondente alla tabella dal modulo corrente
            MibNode tabnode = module.getMibNode(oid_tab);
            //caso in cui la tabella appartiene al modulo corrente
            if (tabnode != null)
                //verifico se si tratta effettivamente di una tabella
                if (tabnode.isTable())
                    return tabnode;
                //caso in cui l'oid non corrisponde ad una tabella
                else
                {
                    trace_err("Wrong OID: "+oid_tab+" not a table");
                    return null;
                }
             //se arrivo qui significa che nel modulo corrente non ho trovato l'oid
            //quindi devo continuare a vedere negli altri moduli supportati
        }

        //se arrivo qui ho considerato tutti i moduli supportati dal motore Snmp
        //specificato, ma non ho trovato in nessuno la tabella
        trace_err("OID "+oid_tab+" not supported from Snmp Engine "+IP+":"+port);
        return null;
    }

	/** recupera il MibNode corrispondente alla tabella tra quelli caricati, se c'e'
	*/
    public MibNode getloadMibTable(String oid_tab)
	{
		//recupero la lista di tutti i moduli correntemente supportati
		Enumeration modules = mibops.getMibModules();
		
		if (modules.hasMoreElements() == false)
        {
            trace_err("No MIB modules loaded");
            return null;
        }

		for( ; modules.hasMoreElements();)
		{
			MibModule mod = (MibModule)modules.nextElement();
			//recupera il nodo corrispondente alla tabella dal modulo corrente
            MibNode tabnode = mod.getMibNode(oid_tab);
            //caso in cui la tabella appartiene al modulo corrente
            if (tabnode != null)
                //verifico se si tratta effettivamente di una tabella
                if (tabnode.isTable())
                    return tabnode;
                //caso in cui l'oid non corrisponde ad una tabella
                else
                {
                    trace_err("Wrong OID: "+oid_tab+" not a table");
                    return null;
                }

             //se arrivo qui significa che nel modulo corrente non ho trovato l'oid
            //quindi devo continuare a vedere negli altri moduli supportati
        }

        return null;
	}

	public MibNode getloadMibTable(PlaceID place,int port,String oid_tab)
	{
       String host = PlaceToHost(place);

    	return(getloadMibTable(host,port,oid_tab));
    }

	/**recupera i Mib supportati dall'agente Snmp corrispondente ad (IP:port)
	*  e carica i moduli corrispondenti se non ancora caricati
	*  IP e' l'indirizzo Ip dell'agente
	*  port e' la porta d'ascolto dell'agente
	*  restituisce la lista dei moduli MIB caricati
	*/
	public MibModule[] getloadSuppMIB(String IP,int port)
	{
		String supported[] = MIBsNodes.getEntry(IP,port);

        //caso in cui non c'e' un'entry corrispondente nella tabella dei MIB supportati
        if (supported == null)
        {
            trace_err("Table of supported MIB modules doesn't contain an entry for: ("+IP+":"+port+")");
            return null;
        }

		MibModule loaded[] = new MibModule[supported.length];
		int k=0;

        for (int j=0; j<supported.length; j++)
        {
            //recupera il modulo supportato
            MibModule module = mibops.getMibModule(supported[j]);
            //caso in cui il modulo non e' ancora stato caricato
            if (module == null)
            try
            {
                module = mibops.loadMibModule(MIBsNodes.getAbsDir()+supported[j]);
                loaded[k++] = module;
            }
            catch(Exception ex)
            {
                trace_err("Error loading mib: "+ex);
                return null;
            }
            else
            	loaded[k++] = module;
           }

         return loaded;
	}

	/**recupera gli OID numerici a partire da quelli simbolici
	* visto che carica tutti i moduli Mib che da configurazione risultano
	* supportati dall'agente, conviene utilizzarlo quando si hanno piu' oids
	* IP e' l'indirizzo dell'agente Snmp
	* port e' la porta d'ascolto dell'agente
	* oids e' una lista di OIDs simbolici supportati dall'agente
	* sostituisce direttamente nella lista passata come argomento gli OID di cui ha
	* trovato il numerico corrispondente
	* restituisce true se e' riuscito a trasformare tutti gli OID passati in numerici
	*             false altrimenti
	*/
	public boolean getNumericOIDs(String IP,int port,String[] oids)
	{
		int j=0;
		Vector index = new Vector();
		//inserisco in un vettore gli indici dei soli oid simbolici
		for (int k=0 ; k<oids.length; k++,j++)
			if (isSymbolic(oids[k]))
				index.addElement(new Integer(k));
		if (index.size() == 0)
			return true;

		//carico tutti i moduli supportati dall'agente
		MibModule[] modules = getloadSuppMIB(IP,port);
	for (int x=0; x<oids.length; x++)
		System.out.println("**"+oids[x]);
		if (modules == null)
			return(getNumericOIDs(oids));

		//numero di OID da convertire
		int nconv = index.size();
	System.out.println("**"+nconv);
		//ricavo il numerico di ogni OID
		for(int i=0; i<modules.length;i++)
		{
			if (nconv != 0)
			for(int k=0; k<index.size(); k++)
			{
				int ind = ((Integer)index.elementAt(k)).intValue();
				//prevedo anche una forma simbolica relativa
				//N.B. in tal caso premetto il prefisso standard (quello del mibii)
				if (oids[ind].startsWith(".") == false)
					oids[ind] = standard_sym_prefix+oids[ind];
				System.out.println(oids[ind]);
				MibNode node = modules[i].getMibNode(oids[ind]);
				if (node != null)
				{
					String conv = node.getNumberedOIDString();
					if (oids[ind].endsWith("0"))
						oids[ind] = conv+".0";
					else
						oids[ind] = conv;
					System.out.println(oids[ind]);
					--nconv;
				}

			 }
		}
		if (nconv != 0)
		{
			trace_err("Impossible to convert all symbolic OID arguments");
			return false;
		}
		else
			return true;
	}

	public boolean getNumericOIDs(PlaceID place, int port, String[] oids)
	{
	System.out.println("## conv");
		return(getNumericOIDs(PlaceToHost(place),port,oids));
	}

	//mi restituisce gli OIDs numerici corrispondenti a quelli simbolici passati
	//cercando nei moduli MIB caricati
	//restituisce true se trova il corrispondente false altrimenti
	public boolean getNumericOIDs(String[] oids)
	{	
		int j=0;
		Vector index = new Vector();
		//inserisco in un vettore gli indici dei soli oid simbolici
		for (int k=0 ; k<oids.length; k++,j++)
			if (isSymbolic(oids[k]))
				index.addElement(new Integer(k));

		if (index.size() == 0)
			return true;

	for (int x=0; x<oids.length; x++)
		System.out.println("@@"+oids[x]);
		//carico tutti i moduli supportati
		Enumeration modules = mibops.getMibModules();
		if (modules.hasMoreElements() == false)
		{
			trace_err("No MIB modules loaded");
			trace_err("Impossible to convert symbolic OID arguments");
			return false;
		}
		
		//numero di OID da convertire
		int nconv = index.size();
	System.out.println("@@"+nconv);
		//ricavo il numerico di ogni OID
		for( ; modules.hasMoreElements();)
		{
			MibModule mod = (MibModule)modules.nextElement();
			if (nconv != 0)
			for(int k=0; k<index.size(); k++)
			{
				int ind = ((Integer)index.elementAt(k)).intValue();
				//prevedo anche una forma simbolica relativa
				if (oids[ind].startsWith(".") == false)
					oids[ind] = standard_sym_prefix+oids[ind];
				System.out.println(oids[ind]);
				MibNode node = mod.getMibNode(oids[ind]);
				if (node != null)
				{
					String conv = node.getNumberedOIDString();
					if (oids[ind].endsWith("0"))
						oids[ind] = conv+".0";
					else
						oids[ind] = conv;
					System.out.println(oids[ind]);
					--nconv;
				}

			 }
		}

		if (nconv != 0)
		{
			trace_err("Impossible to convert all symbolic OID arguments");
			return false;
		}
		else
			return true;

	}

    /**verifica se una stringa corrispondente ad un OID e' simbolica o numerica
	*  oid e' la stringa che rappresenta l'oid in formato numerico o simbolico
	*  restituisce true se il formato e' simbolico
	*              false se e' numerico
	*/
	boolean isSymbolic(String oid)
	{
		StringTokenizer st = new StringTokenizer(oid,".",false);
		try
		{
			Integer.parseInt(st.nextToken());
		}
		//se mi da' eccezione di formato significa che la stringa non rappresenta
		//un intero => formato simbolico
		catch(NumberFormatException en)
		{
				 return true;
		}
		catch(NoSuchElementException et)
		{	}
		return false;
	}

    //metodo di output del contenuto di una PDU tramite ricorso ai MIB files
    //caricati (se la var non corrisp. ad un modulo caricato stampa le info. generiche)
  	public void outPDU(SnmpPDU pdu)
    {
    	if (pdu == null)
    		return;
    	Vector varbinds = pdu.getVariableBindings();
    	for(Enumeration e = varbinds.elements(); e.hasMoreElements();)
    	{
    		SnmpVarBind varb = (SnmpVarBind) e.nextElement();
    		SnmpOID oid = varb.getObjectID();
    		MibNode node = mibops.getMibNode(oid);
    		LeafSyntax leaf = mibops.getLeafSyntax(oid);
    		String sym = node.toString()+"  ";
    		if (node == null)
    			trace(sym+varb.toTagString());
    		else
    		{
    			SnmpVar v = varb.getVariable();
    			byte type = v.getType();
    			if (type == SnmpAPI.INTEGER)
	        	{
    	        	Integer val = (Integer) v.toValue();
	        	    String label = leaf.getLabel(val.intValue());
    		        if (label != null)
            		    trace(sym+v.toTagString()+"("+label+")");
            		else
                		trace(sym+v.toTagString());
		        }
      		    else if ((leaf.toString()).equals("PhysAddress"))
     		    {
 		           SnmpString s = (SnmpString)v;
		           trace(sym+s.getTypeString()+":"+s.toByteString());
		        }
				else    trace(sym+v.toTagString());
			}
		}
    }


    public void outPDU(String[] OIDs,SnmpVar[] vars)
    {
    	SnmpOID[] oids = new SnmpOID[OIDs.length];
    	for (int i=0; i<OIDs.length; i++)
    		oids[i] = new SnmpOID(OIDs[i]);

    	for(int k=0; k<vars.length; k++)
    	{
    		//se non c'e' un valore corrispondente per la variabile significa che l'agente Snmp
    		//non lo implementava
    		if (vars[k] == null)
    		{
    			trace(OIDs[k]+": not available");
    			break;
    		}
    		MibNode node = mibops.getMibNode(oids[k]);
    		LeafSyntax leaf = mibops.getLeafSyntax(oids[k]);
    		String sym = node.toString()+"  ";
    		if (node == null)
    			trace(sym+OIDs[k]+" "+vars[k].toTagString());
    		else
    		{
    			byte type = vars[k].getType();
    			if (type == SnmpAPI.INTEGER)
	        	{
    	        	Integer val = (Integer) vars[k].toValue();
	        	    String label = leaf.getLabel(val.intValue());
    		        if (label != null)
            		    trace(sym+OIDs[k]+" "+vars[k].toTagString()+"("+label+")");
            		else
                		trace(sym+OIDs[k]+" "+vars[k].toTagString());
		        }
      		    else if ((leaf.toString()).equals("PhysAddress"))
     		    {
 		           SnmpString s = (SnmpString)vars[k];
		           trace(sym+OIDs[k]+" "+s.getTypeString()+":"+s.toByteString());
		        }
				else    trace(sym+OIDs[k]+" "+vars[k].toTagString());
			}
		}
    }

    /**recupera gli OID numerici delle colonne della tabella corrispondente
    *  al nodo MIB specificato
    *  tabnode e' il nodo MIB corrispondente alla tabella
    *  restituisce la lista degli OID numerici relativi alle colonne della tabella
    *              null se non si tratta di una tabella
    */
    public String[] getColTable(MibNode tabnode)
    {
        String cols[] ;

        //verifico se si tratta effettivamente di una tabella
        if (tabnode.isTable())
        {
            Vector items = new Vector();
            items = tabnode.getTableItems();
            cols = new String[items.size()];
            int k =0;
            for (Enumeration e=items.elements();e.hasMoreElements();)
            {
                String temp = (String)e.nextElement();
                String numbered = mibops.getMibNode(temp).getNumberedOIDString();
                //provare con String numbered = module.translateToNumbers(temp)
                cols[k++] = numbered;
            }
            return cols;
        }
        //caso in cui l'oid non corrisponde ad una tabella
        else
        {
            trace_err("Wrong OID "+tabnode.getOIDString()+" not a table");
            return null;
        }

    }

	public String[] getColTable(String oid_tab)
	{
		MibNode tabnode = getloadMibTable(oid_tab);
		return(getColTable(tabnode));
	}

    //carica i moduli MIB suportati dal nodo fino a che non trova quello che contiene
    //la tabella specificata tramite il suo oid
    //IP e' l'indirizzo del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //oid_tab e' l'oid della tabella
    //restituisce la lista degli oid completi e in formato numerico delle colonne della tabella in formato numerico
    //null se l'oid passato come argomento non corrisponde ad una tabella o se ci sono
    //stati problemi di caricamento
    public String[] getColTable(String IP,int port,String oid_tab)
    {
		MibNode tabnode = getloadMibTable(IP,port,oid_tab);
		return(getColTable(tabnode));
	}

	/**recupera le informazioni corrispondenti al place
    *place e' il PlaceID del place di interesse
    *restituisce la struttura PlaceInfo contenente tutte le informazioni associate
    *al place
    */
    public PlaceInfo getInfo(PlaceID place)
    {
        PlaceInfo pinfo = PNS.getPlace(place);
        return pinfo;
    }

    /**restituisce l'IP corrispondente al place sotto forma di Stringa
    */
    public String PlaceToHost(PlaceID place)
    {
        PlaceInfo pinfo = PNS.getPlace(place);
        return(pinfo.host.getHostAddress());
    }

//FUNZIONI di CONFIGURAZIONE dei DOMINI: per ognuno si memorizza lo stato del sottoManager e il suo identificatore

	public boolean setDomainOn(PlaceID domain)
	{
		return (statD.setOn(domain));
	}
	
	public boolean isDomainOn(PlaceID domain)
	{
		return (statD.isOn(domain));
	}
	
	public boolean setDomainOff(PlaceID domain)
	{
		return (statD.setOff(domain));
	}

	public boolean setSubID(PlaceID domain, AgentID id)
	{
		return (statD.setID(domain,id));
	}
	
	public AgentID getSubID(PlaceID domain)
	{
		return (statD.getID(domain));
	}	
	
	public Vector getAllDomainOn()
	{
		return (statD.getAllOn());
	}
	
//FUNZIONI di CONFIGURAZIONE degli AGENTI SNMP
//in realta' si potrebbero usare direttamente i metodi di InfosAgentSnmp sulla variabile
//agentSnmp

   public boolean addHostSnmp(String host,int snmp_port,int snmp_version,String security_name,PlaceID place)
    {
        return(agentSnmp.addEntry(host,snmp_port,snmp_version,security_name,place));
    }

    public boolean delHostSnmp(String host,int port)
    {
        return(agentSnmp.removeEntry(host,port));
    }

    public void modifyHostSnmp(String host, int snmp_port, int snmp_version,String security_name,PlaceID place)
    {
        agentSnmp.modifyEntry(host,snmp_port,snmp_version,security_name,place);
    }

    public void showHostsConfiguration()
    {
       	agentSnmp.showContent();
    }

    public String getSecurityName(String host,int port)
    {
        return(agentSnmp.getSecurityName(host,port));
    }

    public int getAgentVersion(String host, int port)
    {
    	return(agentSnmp.getSupportedVersion(host,port));
    }

	public PlaceID getAgentPlace(String host, int port)
	{
		return(agentSnmp.getCorrespondingPlace(host,port));
	}
	
	public int[] getAgentsPort(PlaceID[] places)
	{
		return (agentSnmp.getCorrespondingPorts(places));
	}

    /**deserializza la configurazione corrente, cioe' ricopia nelle
    * strutture USMUserTable,SnmpEngineTable,InfoPlaceSnmp,SupportedMIBsNodes
    * quanto precedentemenente memorizzato su file
    * N.B. cancella il contenuto attuale delle strutture
    */
	public boolean restoreConfiguration()
	{
		//prima controllo se e' gia' stata effettuata la deserializzazione da quando e'
		//stato avviato l'ultima volta
		if (deserialized)
			return true;
       	api.setSerializeFileName(USMserFile);
        try
   	    {
       	    if (api.deSerialize() == false)
       	    {
           	    trace_err("Error restoring V3 Configuration Tables from: "+api.getSerializeFileName());
           	    return false;
           	}
            else
   	        {
       	        userTable = api.getUSMTable();
           	    engineTable = api.getSnmpEngine();
           	}
           	if (new File("EngineEntry.ser").exists())
               	engineTable.deSerialize();
            if ((MIBsNodes.deSerialize()== false) || (agentSnmp.deSerialize() == false))
            {
   	            trace_err("Error restoring SupportedMIBNodes or InfosAgentSnmp");
   	            return false;
   	        }
       	}
       	catch(SnmpException e)
       	{
       		trace_err("Exception during deserialization");
       		e.printStackTrace();
       		return false;
       	}

       	deserialized = true;

       	return true;
    }

    /**serializza la configurazione corrente, cioe' rende permanente il contenuto delle
    * strutture USMUserTable,SnmpEngineTable,InfoPlaceSnmp,SupportedMIBsNodes
    * N.B. sovrascrive la configurazione precedentemente serializzata
    */
    public boolean saveConfiguration()
    {
    	if (api.getSerializeFileName() == null)
    		api.setSerializeFileName(USMserFile);
	    try
        {
    	    if (api.serialize()== false)
    	    {
        	    trace_err("Error storing V3 Configuration Tables");
        	    return false;
        	}
            engineTable.serialize();
            MIBsNodes.serialize();
            //placeSnmp.serialize();
            agentSnmp.serialize();
        }
        catch(SnmpException e) {trace_err("Exception during serialization");
                                     return false;}
		return true;
    }


    //stampa il contenuto della tabella accompagnato dagli oid simbolici
    public void printTable(PrintStream o, snmpTable tab)
    {
        if (tab == null)
            return;
        tab.printTable(o,mibops);
        return;
    }

}