package MonitorConfig;

import java.util.*;
import java.io.*;
import java.net.*;
import com.adventnet.snmp.snmp2.SnmpAPI;
import SOMA.naming.*;
import SOMA.naming.place.*;
import SOMA.*;

public class InfosPlaceSnmp extends Debug implements Serializable
{
	//hashtable di entry avente per chiave la coppia (placeID:porta)
	//perche' su uno stesso place potrebbero esserci piu' agenti Snmp in ascolto su porte diverse
    Hashtable infos = new Hashtable();
    //introduco come attributo anche file di serializzazione prevedendo
    //di poter utilizzare un file diverso per ogni place
    File serFile = null;

    public InfosPlaceSnmp(String file,boolean deb, PrintStream o)
    {
        super(deb,o);
        try {serFile = new File(file);}
        catch(NullPointerException ne) {trace_err("Attemp to create a file without its name"); return;}
    }

    //metodo che costruisce le associazioni agente-place cioe' (host:porta) --> placeID
	//utile se voglio sapere su che place si trova un agente, perche' su un host potrei
	//avere places differenti
	Hashtable hostToPlace(Environment env)
	{
		if (infos == null)
			return null;
		Hashtable result = new Hashtable();
		//estraggo ogni chiave, ne ricavo host e porta, creo un ipaddress che sara' la chiave
		//della mia nuova hashtable
		for(Enumeration e = infos.keys(); e.hasMoreElements();)
		{
			key k = (key)e.nextElement();
			PlaceID p =  k.place;
			int port = k.snmp_port;
			//mi rivolgo al pns per ottenere le info. associate ad un certo placeID
    	    PlaceInfo pinfo = env.placeNameService.getPlace(p);
    	    InetAddress host = pinfo.host;
    	    IpAddress newk = new IpAddress(host,port);
    	    try
        	{
            	result.put(newk,p);
        	}
        	catch (NullPointerException ex){ }

		}

		return result;
	}

	//restituisce la lista di PlaceID corrispondenti alle coppie (host:porta)
	public PlaceID[] toPlaces(String[] hosts, int[] ports, Environment env)
	{
		int Nhosts = hosts.length;
		PlaceID[] ret = new PlaceID[Nhosts];
		Hashtable association = hostToPlace(env);
		for(int i=0; i<Nhosts; i++)
		{
			ret[i] = (PlaceID) association.get(new IpAddress(hosts[i],ports[i]));
			if (ret[i] == null)
			{
				trace_err("PlaceID corresponding to: ("+hosts[i]+":"+ports[i]+") absent");
				return null;
			}
		}
		return ret;
	}

    boolean addEntry(key k,entry e)
    {
        if (infos.containsKey(k))
        {
            trace_err("Esiste gia' un'entry corrispondente a: "+k.toString());
            return false;
        }
        else
        try
        {
            infos.put(k,e);
        }
        catch (NullPointerException ex){ return false;}
        return true;
    }

    public boolean addEntry(PlaceID place, int port, int version,String sec_name)
    {
    	key k = new key(place,port);
        entry e = new entry(version,port,sec_name);
        return(addEntry(k,e));
    }

    public boolean addEntry(PlaceID place,int version,String sec_name)
    {
    	key k = new key(place);
        entry e = new entry(version,sec_name);
        return(addEntry(k,e));
    }

    public boolean addEntry(PlaceID place)
    {
    	key k = new key(place);
        entry e = new entry();
        return(addEntry(k,e));
    }

    public boolean removeEntry(PlaceID place,int port)
    {
    	key k = new key(place,port);
        if (infos.remove(k) == null)
        {
            trace_err("Non esiste un'entry corrispondente a "+place.toString()+":"+port);
            return false;
        }
        return true;
    }

    public void modifyEntry(key k,entry e)
    {
        if (infos.containsKey(k))
            infos.remove(k);
        infos.put(k,e);
        return;
    }

    public void modifyEntry(PlaceID place,int port, int version,String sec_name)
    {
    	key k = new key(place,port);
        entry e = new entry(version,sec_name);
        modifyEntry(k,e);
    }

    public void modifyEntry(PlaceID place,int version,String sec_name)
    {
    	key k = new key(place);
        entry e = new entry(version,sec_name);
        modifyEntry(k,e);
    }

    public void modifyEntry(PlaceID place)
    {
    	key k = new key(place);
        entry e = new entry();
        modifyEntry(k,e);
    }

    public void modifyEntry(PlaceID place,int port)
    {
    	key k = new key(place,port);
        entry e = new entry();
        modifyEntry(k,e);
    }

	public int getSupportedVersion(PlaceID place)
    {
		return(getSupportedVersion(place,161));
    }

    public int getSupportedVersion(PlaceID place,int port)
    {
    	key k = new key(place,port);
    	entry e = (entry)infos.get(k);
    	if (e == null)
    		return -1;
    	return (e.getVersion());
    }

    public String getSecurityName(PlaceID place,int port)
    {
    	key k = new key(place,port);
        entry e = (entry)infos.get(k);
        if (e == null)
        	return null;
        return(e.getSecName());
    }

    public String getSecurityName(PlaceID place)
    {
        return(getSecurityName(place,161));
    }

    public boolean setSerializeFileName(String file)
    {
		try{ serFile = new File(file); }
		catch(NullPointerException ex) {trace_err("Attempt to set serialize file without its name"); return false;}
        return true;
    }

    public String getSerializeFileName()
    {
        return (serFile.getAbsolutePath());
    }

    public void showContent()
    {
    	out.println("(PlaceID:SnmpPort)  SecName  SnmpVersion");
    	for (Enumeration e = infos.keys(); e.hasMoreElements();)
    	{
    		key curkey = (key)e.nextElement();
    		out.print(curkey.toString()+" ");
    		entry cur = (entry)infos.get(curkey);
    		cur.show(out);
	   	}
    }

    public void serialize()
    {
        try
        {
            FileOutputStream fout = new FileOutputStream(serFile);
            ObjectOutputStream out = new ObjectOutputStream(fout);
            out.writeObject(this);
        }
        catch(Exception e) { }

    }

    public boolean deSerialize()
    {
        try
        {
        	//se il file esiste ed e' leggibile
        	if (serFile.canRead())
	        try
    	    {
        	    FileInputStream fin = new FileInputStream(serFile);
            	ObjectInputStream in = new ObjectInputStream(fin);
           	    InfosPlaceSnmp copy = (InfosPlaceSnmp) in.readObject();
            	infos = copy.infos;
            	//in realta' questi due campi gia' dovrebbero coincidere
		        serFile = copy.serFile;
	            return true;
    	    }
        	catch(Exception e) {trace_err("Problems occurred during deserialization");}
        }
        catch(SecurityException ex)
        {
        	String abs = serFile.getAbsolutePath();
        	trace_err("Permission to read deserialization file "+abs+" denied on this place");
        }

        return false;
    }

    class key implements Serializable
    {

    	PlaceID place;
    	int snmp_port;

    	key(PlaceID p)
    	{
    		place = p;
    		snmp_port = 161;
    	}

    	key(PlaceID p, int port)
    	{
    		place = p;
    		snmp_port = port;
    	}

    	public String toString()
    	{
    		return("("+place.toString()+":"+snmp_port+")");
    	}

    }

    class entry implements Serializable
    {
        int snmp_version;
        String sec_name;

        entry()
        {
            snmp_version = SnmpAPI.SNMP_VERSION_1;
            sec_name = "public";
        }

        entry(int version, String name)
        {
            snmp_version = version;
            sec_name = name;
        }

        entry(int version, int port, String name)
        {
            snmp_version = version;
            sec_name = name;
        }

        public int getVersion()
        {
            return snmp_version;
        }

        public String getSecName()
        {
            return sec_name;
        }

		public void show(PrintStream out)
        {
        	String version = null;

        	if (snmp_version == SnmpAPI.SNMP_VERSION_1)
        		version = "v1";
        	else if (snmp_version == SnmpAPI.SNMP_VERSION_2C)
        		version = "v2C";
        	else if (snmp_version == SnmpAPI.SNMP_VERSION_3)
        		version = "v3";
        	out.println(sec_name+"	"+version);
        }
    }

}