package MonitorConfig;

import java.util.*;
import java.io.*;

//N.B. CAMBIA IL MODO DI CREARE IL NOME DEL FILE (adesso dipende dal sistema)
public class SupportedNodesMIB extends Debug implements Serializable
{
    //tabella in cui alla chiave (IP:porta) viene associata una lista di nomi di
    //moduli MIB supportati dal motore Snmp corrispondente
    Hashtable Mibs = new Hashtable();
    String serFile = null;

    //nome assoluto del direttorio locale nel quale si trovano i MIB files corrispondenti
    //ai moduli MIB, nella forma <path assoluto>\ o <path assoluto>/ !!!!!
    String abs_dir;

    SupportedNodesMIB(String file,boolean deb,PrintStream o)
    {
        super(deb,o);
        Mibs = new Hashtable();
        abs_dir = new String(".//");
        serFile = file;
    }

    public SupportedNodesMIB(String dir,String file,boolean deb,PrintStream o)
    {
        this(file,deb,o);
        abs_dir = new String(dir+"//");
    }

    //resituisce il nome assoluto del direttorio nel quale trovare i MIB files
    //es. "c:\snmp\mibs\"
    public String getAbsDir()
    {
        return (abs_dir);
    }


    void showContent()
    {
    	Enumeration e = Mibs.elements();
    	out.println("IP:porta nome_MIB1 nome_MIB2 .. nome_MIBk");
		for(Enumeration k = Mibs.keys();k.hasMoreElements();)
		{
			out.print(((IpAddress)k.nextElement()).toString()+"  ");
			String[] cur = (String[])e.nextElement();
			for(int i=0; i<cur.length;i++)
				out.print(cur[i]+"  ");
			out.println("");
		}

	}

    //setta il nome assoluto del direttorio nel quale trovare i MIB files
    //es. "c:\snmp\mibs\"
    public boolean setAbsDir(String path)
    {
        File dir = new File(path);
        if (dir.isDirectory())
        {
            //abs_dir = path+"//";
            abs_dir = path;
            return true;
        }
        System.out.println("MIBs dir: "+abs_dir);
        return false;
    }

    //fornisce un entry della tabella dei Mibs supportati dai nodi
    //Ip e' l'indirizzo IP del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //restituisce la lista dei nomi di moduli MIB supportati dal motore Snmp corrispondente
    //a IP:port, null se non esiste un'entry corrispondente
    public String[] getEntry(String IP, int port)
    {
        //System.out.println(IP+":"+port);
        return ((String[])Mibs.get(new IpAddress(IP,port)));
    }


    /*metodo per l'aggiunta di una nuova entry alla tabella dei mibs supportati
    //Ip e' l'indirizzo IP del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //mibs e' il vettore dei nomi dei mibs supportati dal motore Snmp
    //restituisce true se l'entry e' stata aggiunta con successo
    //            false se e' gia' presente un'entry con questa chiave (Ip:porta)
    */
    public boolean addEntry(String IP, int port, String mibs[])
    {
        IpAddress key = new IpAddress(IP,port);
        if (Mibs.containsKey(key))
        {
            trace_err("Esiste gia' un'entry corrispondente a: "+IP+":"+port);
            return false;
        }
        else
        try
        {
            Mibs.put(key,mibs);
        }
        catch (NullPointerException e){ return false;}
        return true;
    }

    /*rimuove dalla tabella dei mib supportati l'entry corrispondente ad IP:porta
    //Ip e' l'indirizzo IP del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //restituisce true se ha rimosso con successo l'entry
    //            false se l'entry non esiste
    */
    public boolean removeEntry(String IP,int port)
    {
        if (Mibs.remove(new IpAddress(IP,port)) == null)
        {
            trace_err("Non esiste un'entry corrispondente a "+IP+":"+port);
            return false;
        }
        return true;
    }

    /* modifica un'entry gia' esistente sostituendo la lista dei mib supportati
    // con una nuova lista
    //Ip e' l'indirizzo IP del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //new_mibs e' la nuova lista di mibs supportati
    */
    public void modifyEntry(String IP, int port, String new_mibs[])
    {
        IpAddress key = new IpAddress(IP,port);
        if (Mibs.containsKey(key))
            Mibs.remove(key);
        Mibs.put(key,new_mibs);
        return;
    }

    /*aggiunge un MIB file alla lista di quelli supportati dal motore su IP:port
    //Ip e' l'indirizzo IP del motore Snmp
    //port e' la porta d'ascolto del motore Snmp
    //new_mib e' il nome del nuovo modulo MIB
    */
    void addMib(String IP, int port, String new_mib)
    {
        IpAddress key = new IpAddress(IP,port);
        String curval[];
        String newval[];
        if ((curval = (String[])Mibs.get(key))!= null)
        {
            int curlen = curval.length;
            newval = new String[++curlen];
            newval = curval;
            newval[curlen] = new_mib;
            modifyEntry(IP,port,newval);
        }
        else
        {
            newval = new String[1];
            newval[0] = new_mib;
            addEntry(IP,port,newval);
        }
    }

    void removeMib(String IP, int port, String old_mib)
    {
        IpAddress key = new IpAddress(IP,port);
        String curval[];
        String newval[];
        if ((curval = (String[])Mibs.get(key))!= null)
        {
            int curlen = curval.length;
            newval = new String[--curlen];
            newval = curval;
            modifyEntry(IP,port,newval);
        }
        else
            trace("Non esiste un'entry corrispondente a "+IP+":"+port);
        return;
    }

    public boolean setSerializeFileName(String file)
    {
        serFile = file;
        return true;
    }

    public String getSerializeFileName()
    {
        return serFile;
    }

    public void serialize()
    {
        if (serFile == null)
        {
            trace_err("Supported MIBs serialization file not set");
            return;
        }
        try
        {
            FileOutputStream fout = new FileOutputStream(serFile);
            ObjectOutputStream out = new ObjectOutputStream(fout);
            out.writeObject(this);
        }
        catch(Exception e) {}
    }

    public boolean deSerialize()
    {
        if (serFile == null)
        {
            trace_err("Supported MIBs deserialization file not set");
            return false;
        }
        if (new File(serFile).exists())
        try
        {
            FileInputStream fin = new FileInputStream(serFile);
            ObjectInputStream in = new ObjectInputStream(fin);
            SupportedNodesMIB copy = (SupportedNodesMIB) in.readObject();
            Mibs = copy.Mibs;
            serFile = copy.serFile;
            return true;
        }
        catch(Exception e) {}

        return false;
    }

}