package MonitorConfig;

import java.util.*;
import java.io.*;
import com.adventnet.snmp.snmp2.*;
import com.adventnet.snmp.mibs.*;
import java.io.Serializable;
import StaticMonitoring.sharedOut;

public class snmpTable extends Debug implements Serializable
{
    //gli indici di riga per me vanno da 1 a Nrows
    int Nrows;
    int Ncolumns;
    String prefix;

    //vettore contenente elementi snmpRow corrispondenti alle varie righe
    Vector tab;

    public snmpTable(String oid,int Ncol,boolean deb,PrintStream o)
    {
        super(deb,o);
        prefix = new String(oid);
        Nrows = 0;
        Ncolumns = Ncol;
        tab = new Vector();
    }

    public snmpTable(String oid,int rows,int cols,snmpRow el[],boolean deb,PrintStream o)
    {
        this(oid,cols,deb,o);
        Nrows = rows;
        for (int i=0; i<rows; i++)
            tab.addElement(el[i]);
    }

	//ridefinisco il metodo della superclasse Debug cosi' che se viene cambiato
	//l'output stream della tabella allo stesso modo deve cambiare quello di ogni riga gia'
	//presente nella tabella
	public void setOut(PrintStream o)
	{
		super.setOut(o);
		Enumeration e = tab.elements();
		for (int j=0; e.hasMoreElements(); j++)
			((snmpRow)e.nextElement()).setOut(o);
	}

    public int getNRows()
    {
        return Nrows;
    }

    public int getNCols()
    {
        return Ncolumns;
    }

    //permette di accedere ad una determinata riga (indici da 1 a Nrighe)
    //index e' l'indice di riga: da 1 a Nrighe
    //restituisce la riga sotto forma di snmpRow
    public snmpRow getRow(int index)
    {
        return ((snmpRow)tab.elementAt(index-1));
    }

    //aggiunge una riga alla tabella assegnandole il primo indice libero
    //elems_row e' la lista degli elementi della riga di tipo SnmpVar
    public void addRow(SnmpVar elems_row[])
    {
        Nrows++;
        tab.addElement(new snmpRow(Ncolumns,(new Integer(Nrows)).toString(),elems_row,debug,out));
    }

    public void addRow(SnmpPDU pdu_row)
    {
        Nrows++;
        tab.addElement(new snmpRow(Ncolumns,(new Integer(Nrows)).toString(),pdu_row,debug,out));
    }

    //permette di ottenere uno specifico elemenento di tabella
    //row e' l'indice di riga dell'elemento: da 1 a Nrighe
    //col e' l'indice di colonna dell'elemento: da 1 a Ncolonne
    //restituisce l'elemento di tipo SnmpVar nella posizione (row,col) della tabella
    public SnmpVar getElement(int row, int col)
    {
        snmpRow current = ((snmpRow)tab.elementAt(--row));
        return(current.getElement(col));
    }

    //permette di ottenere l'oid globale di un elemento
    //row e' l'indice di riga dell'elemento: da 1 a Nrighe
    //col e' l'indice di colonna dell'elemento: da 1 a Ncolonne
    //restituisce un oid globale : es. per l'elemento (1,4) se la tabella ha prefisso
    //1.4.10 sara' 1.4.10.1.4.1 (dove 1 corrisponde al nodo che rappresenta l'entry, 4 alla
    //colonna e 1 alla riga, cioe' alla particolare istanza dell'entry)
    public String getTotalOID(int row,int col)
    {
        return(prefix+".1."+col+"."+row);
    }

	public String getTabOID()
	{
		return prefix;
	}

    //stampa il contenuto della tabella accompagnato dagli oid numerici
    synchronized public void printTable(PrintStream o)
    {
        o.println("TABLE "+prefix);
        for(Enumeration e=tab.elements();e.hasMoreElements();)
        {
            snmpRow row = (snmpRow)e.nextElement();
            o.println("ROW "+row.rowIndex());
            row.printRow(new sharedOut(o));
        }
    }

    //stampa il contenuto della tabella accompagnato dagli oid simbolici
    synchronized public void printTable(PrintStream o,MibOperations mibops)
    {
        for (int j=0; j < Nrows; j++)
        {
            out.println("ROW "+(j+1));
            for(int i = 0; i< Ncolumns; i++)
            {
                //ricostruisco l'oid corrispondente all'elem. corrente
                SnmpOID oid = new SnmpOID(getTotalOID(j+1,i+1));
                //recupero il MibNode corrisp. all'oid
                MibNode node = mibops.getMibNode(oid);
                LeafSyntax leaf = mibops.getLeafSyntax(oid);
                //System.out.println(leaf.getName());
                //o.print(oid+"  "+node.toString()+"  ");
                o.print(node.toString()+"  ");
                printElem(new sharedOut(o),j+1,i+1,leaf);
            }
        }
        out.println("Printable: "+Nrows);
        return;
    }

    //sempre indici da 1 a N
    public void printElem(PrintStream o,int row,int col,LeafSyntax leaf)
    {
        snmpRow riga = (snmpRow)tab.elementAt(--row);
        riga.printElem(o,col,leaf);
    }

}
